==> sptk-3.5.7.14.tbz2: sptk-3.5.7.14/src/themes/CThemesGtk.cpp <==
/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CThemesGtk.cpp  -  description
                             -------------------
    begin                : Thu May 22 2008
    copyright            : (C) 2003-2008 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

#include <sptk3/gui/CThemeImageCollection.h>


/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/sptk.h>
#include <sptk3/cutils>
#include <sptk3/cgui>

#include "ThemeUtils.h"
#include "CGtkThemeLoader.h"

#ifdef _WIN32
    #include <winsock2.h>
    #include <windows.h>
#endif

using namespace std;
using namespace sptk;

void CThemes::loadGtkButton(CXmlNode* imageNode,std::map<CThemeButtonState,std::string>& buttonFileNames) {
    static const CStrings buttonStates("NORMAL|ACTIVE|DFRAME|PRELIGHT","|"); /// DFRAME is a stub

    bool defaultFrame = imageNode->getAttribute("detail","") == "buttondefault";

    string fileName = imageNode->getAttribute("file");
    if (fileName.empty())
        fileName = imageNode->getAttribute("overlay_file");

    string state = upperCase(imageNode->getAttribute("state","NORMAL"));
    string border = imageNode->getAttribute("border");
    string shadow = upperCase(imageNode->getAttribute("shadow","OUT"));
    if (shadow == "ETCHED_IN")
        return;
    if (fileName[0] == '/') fileName = fileName.substr(1,255);
    int buttonState = buttonStates.indexOf(state);
    if (defaultFrame)
        buttonState = THM_DEFAULT_FRAME;
    else if (buttonState > -1 && shadow == "IN")
            buttonState++;
    if (buttonState > -1 && fileName.find(".png") != string::npos)
        buttonFileNames[CThemeButtonState(buttonState)] = m_themeFolder + fileName;
}

void CThemes::loadGtkButtonFileNames(CXmlDoc& xml,string XPath,map<CThemeButtonState,string>& buttonFileNames,string orientation) {
    CXmlNodeVector buttonImages;
    
    buttonFileNames.clear();
    xml.select(buttonImages,XPath);
    for (CXmlNode::iterator itor = buttonImages.begin(); itor != buttonImages.end(); itor++) {
        CXmlNode* imageNode = *itor;
        if (!orientation.empty() && imageNode->getAttribute("arrow_direction") != orientation)
            continue;
        loadGtkButton(imageNode,buttonFileNames);
    }

    if (buttonFileNames[THM_IMAGE_ACTIVE].empty())
        buttonFileNames[THM_IMAGE_ACTIVE] = buttonFileNames[THM_IMAGE_NORMAL];
    
    if (buttonFileNames[THM_IMAGE_NORMAL_HIGHLITED].empty())
        buttonFileNames[THM_IMAGE_NORMAL_HIGHLITED] = buttonFileNames[THM_IMAGE_NORMAL];

    if (buttonFileNames[THM_IMAGE_ACTIVE_HIGHLITED].empty())
        buttonFileNames[THM_IMAGE_ACTIVE_HIGHLITED] = buttonFileNames[THM_IMAGE_ACTIVE];
}

void CThemes::loadGtkButtons(CXmlDoc& xml,string styleName,CThemeImageCollection& buttons) {
    string XPath("/styles/style[@name='" + styleName + "']/engine[@name='pixmap']/image");
    buttons.loadFromGtkTheme(xml,XPath);
}

void CThemes::loadGtkScrollbarButtons(CXmlDoc& xml,string orientation,CThemeImageCollection& buttonImages) {
    map<CThemeButtonState,string> buttonFileNames;
    string XPath("/styles/style[@name='scrollbars']/engine[@name='pixmap']/image[@function='STEPPER']");
    buttonImages.loadFromGtkTheme(xml,XPath,"arrow_direction",orientation);
}

void CThemes::loadGtkScrollbarTroughs(CXmlDoc& xml) {
    static const char* orientation[2] = { "VERTICAL", "HORIZONTAL" };
    for (unsigned i = 0; i < 2; i++) {
        CThemeImageCollection* images = &m_scrollBar[i].m_parts[CScrollBarImages::TROUGH];
        images->loadFromGtkTheme(xml,"/styles/style[@name='scrollbars']/engine[@name='pixmap']/image[@detail='trough']","orientation",orientation[i]);
    }
}

void CThemes::loadGtkScrollbarSliders(CXmlDoc& xml) {
    static const char* orientation[2] = { "VERTICAL", "HORIZONTAL" };

    CThemeImageCollection* images;
    for (unsigned i = 0; i < 2; i++) {
        images = &m_scrollBar[i].m_parts[CScrollBarImages::SLIDER];
        images->loadFromGtkTheme(xml,"/styles/style[@name='scrollbars']/engine[@name='pixmap']/image[@function='SLIDER']","orientation",orientation[i]);
    }
}

void CThemes::loadGtkScrollbars(CXmlDoc& xml) {
    loadGtkScrollbarButtons(xml,"UP",m_scrollBar[THM_SCROLLBAR_VERTICAL].m_steppers[0]);
    loadGtkScrollbarButtons(xml,"DOWN",m_scrollBar[THM_SCROLLBAR_VERTICAL].m_steppers[1]);
    loadGtkScrollbarButtons(xml,"LEFT",m_scrollBar[THM_SCROLLBAR_HORIZONTAL].m_steppers[0]);
    loadGtkScrollbarButtons(xml,"RIGHT",m_scrollBar[THM_SCROLLBAR_HORIZONTAL].m_steppers[1]);
    loadGtkScrollbarTroughs(xml);
    loadGtkScrollbarSliders(xml);
}

void CThemes::loadGtkColors(CXmlNode* textColorsNode) {
    static const CStrings   colorNames("fg_color|bg_color|base_color|text_color|selected_bg_color|selected_fg_color|tooltip_bg_color|tooltip_fg_color","|");
    CStrings colorMapStrings(textColorsNode->getAttribute("value"),"\\n");

    for (unsigned i = 0; i < colorMapStrings.size(); i++) {
        CStrings colorInfo(colorMapStrings[i],":#");
        char* eptr;
        unsigned rgbColor = strtol(colorInfo[1].c_str(),&eptr,16);
        unsigned blue  = rgbColor % 256; rgbColor /= 256;
        unsigned green = rgbColor % 256; rgbColor /= 256;
        unsigned red   = rgbColor % 256;

        map<string,Fl_Color>::iterator citor = m_colorNames.find(colorInfo[0]);
        if (citor == m_colorNames.end())
            continue;
        Fl_Color acolor = citor->second;
        Fl::set_color(acolor,red,green,blue);
    }
}

void CThemes::loadGtkTheme() {
    CGtkThemeParser gtkThemeLoader;
    string          testThemeName("Overglossed");
    gtkThemeLoader.load(testThemeName);

    m_gtkTheme = true;

    m_themeFolder = gtkThemeLoader.themeFolder();

    CBuffer buffer;
    gtkThemeLoader.xml().save(buffer);
    buffer.saveToFile("/tmp/" + testThemeName + ".xml");

    CXmlDoc& xml = gtkThemeLoader.xml();

    /// Find GTK text color info:
    CXmlNodeVector colorNodes;
    xml.select(colorNodes,"/gtk_color_scheme");
    if (colorNodes.size() == 1) {
        CXmlNode* colorSchemaNode = *colorNodes.begin();
        loadGtkColors(colorSchemaNode);
    }

    loadGtkButtons(xml,"button",m_normalButtons);
    loadGtkButtons(xml,"checkbutton",m_checkButtons);
    loadGtkButtons(xml,"radiobutton",m_radioButtons);
    loadGtkButtons(xml,"combobutton",m_comboButtons);

    loadGtkScrollbars(xml);
}

==> sptk-3.5.8.8.tbz2: sptk-3.5.8.8/src/themes/CThemesGtk.cpp <==
/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CThemesGtk.cpp  -  description
                             -------------------
    begin                : Thu May 22 2008
    copyright            : (C) 2003-2008 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/sptk.h>
#include <sptk3/cutils>
#include <sptk3/cgui>

#include "ThemeUtils.h"
#include "CGtkThemeLoader.h"
#include <sptk3/gui/CThemeImageCollection.h>

#ifdef _WIN32
    #include <winsock2.h>
    #include <windows.h>
#endif

using namespace std;
using namespace sptk;

void CThemes::loadGtkButton(CXmlNode* imageNode,std::map<CThemeImageState,std::string>& buttonFileNames) {
    static const CStrings buttonStates("NORMAL|ACTIVE|DFRAME|PRELIGHT","|"); /// DFRAME is a stub

    bool defaultFrame = imageNode->getAttribute("detail","") == "buttondefault";

    string fileName = imageNode->getAttribute("file");
    if (fileName.empty())
        fileName = imageNode->getAttribute("overlay_file");

    string state = upperCase(imageNode->getAttribute("state","NORMAL"));
    string border = imageNode->getAttribute("border");
    string shadow = upperCase(imageNode->getAttribute("shadow","OUT"));
    if (shadow == "ETCHED_IN")
        return;
    if (fileName[0] == '/') fileName = fileName.substr(1,255);
    int buttonState = buttonStates.indexOf(state);
    if (defaultFrame)
        buttonState = THM_DEFAULT_FRAME;
    else if (buttonState > -1 && shadow == "IN")
            buttonState++;
    if (buttonState > -1 && fileName.find(".png") != string::npos)
        buttonFileNames[CThemeImageState(buttonState)] = m_themeFolder + fileName;
}

void CThemes::loadGtkButtonFileNames(CXmlDoc& xml,string XPath,map<CThemeImageState,string>& buttonFileNames,string orientation) {
    CXmlNodeVector buttonImages;
    
    buttonFileNames.clear();
    xml.select(buttonImages,XPath);
    for (CXmlNode::iterator itor = buttonImages.begin(); itor != buttonImages.end(); itor++) {
        CXmlNode* imageNode = *itor;
        if (!orientation.empty() && imageNode->getAttribute("arrow_direction") != orientation)
            continue;
        loadGtkButton(imageNode,buttonFileNames);
    }

    if (buttonFileNames[THM_IMAGE_ACTIVE].empty())
        buttonFileNames[THM_IMAGE_ACTIVE] = buttonFileNames[THM_IMAGE_NORMAL];
    
    if (buttonFileNames[THM_IMAGE_NORMAL_HIGHLITED].empty())
        buttonFileNames[THM_IMAGE_NORMAL_HIGHLITED] = buttonFileNames[THM_IMAGE_NORMAL];

    if (buttonFileNames[THM_IMAGE_ACTIVE_HIGHLITED].empty())
        buttonFileNames[THM_IMAGE_ACTIVE_HIGHLITED] = buttonFileNames[THM_IMAGE_ACTIVE];
}

void CThemes::loadGtkButtons(CXmlDoc& xml,string styleName,CThemeImageCollection& buttons,string function) {
    string XPath("/styles/style[@name='" + styleName + "']/engine[@name='pixmap']/image");
    buttons.loadFromGtkTheme(xml,XPath,"function",function);
}

void CThemes::loadGtkTheme(string gtkThemeName) {
    CGtkThemeParser gtkThemeLoader;
    string          testThemeName(gtkThemeName);

    try {
        gtkThemeLoader.load(testThemeName);
    }
    catch (...) {
        return;
    }

    m_gtkTheme = true;

    m_themeFolder = gtkThemeLoader.themeFolder();

    CBuffer buffer;
    gtkThemeLoader.xml().save(buffer);
    
    try { // Debug
        buffer.saveToFile("/svn/sptk3/trunk/" + testThemeName + ".xml");
    }
    catch (...) {}

    CXmlDoc& xml = gtkThemeLoader.xml();

    /// Load theme colors
    m_colors.loadFromGtkTheme(xml);

    loadGtkButtons(xml,"button",m_normalButtons,"BOX");
    loadGtkButtons(xml,"checkbutton",m_checkButtons,"CHECK");
    loadGtkButtons(xml,"radiobutton",m_radioButtons,"OPTION");
    loadGtkButtons(xml,"combobutton",m_comboButtons,"BOX");

    loadGtkScrollbars(xml);
    
    m_progressBar[0].loadFromGtkTheme(xml,"/styles/style[@name='progressbar']/engine[@name='pixmap']/image[@detail='trough']","orientation","HORIZONTAL");
    m_progressBar[1].loadFromGtkTheme(xml,"/styles/style[@name='progressbar']/engine[@name='pixmap']/image[@detail='bar']","orientation","HORIZONTAL");

    CXmlNodeVector tabImageNodes;
    xml.select(tabImageNodes,"/styles/style[@name='tabs']/engine[@name='pixmap']/image[@function='EXTENSION']");
    if (tabImageNodes.size())
        m_tabImages.loadGtkTabs(tabImageNodes);
}
